Components.utils.import("resource://jetpack/modules/places.js");

const DOMAIN_PREFIX = Date.now();
const TEST_URL = "http://baz." + DOMAIN_PREFIX + ".mozilla.com/";

let PlacesTests = {
  get places() {
    return new PlacesQuery();
  },

  testAddAndFindBookmark: function (self) {
    let bObj = { url: TEST_URL,
                 title: "my mozilla " + DOMAIN_PREFIX,
                 tags: ["baz", "mozilla"]
               };
    function callbackA(createdBookmarks) {
      self.assert(createdBookmarks.length == 1);
      self.assert(createdBookmarks[0].uri == TEST_URL);
    }

    this.places.bookmark.create(bObj, callbackA);

    // find some bookmarks
    function callbackB(bookmarks) {
      console.log(bookmarks.length + " bookmarks found\n");
      let len = bookmarks.length;
      self.assert(len > 0);
      for (let i = 0;  i < len; i++) {
        self.assert(bookmarks[i].uri != undefined);
      }
    }

    let qObj = { phrase: "mozilla", where: "bookmarks" };
    this.places.find(qObj, callbackB);

    // create an array of bookmarks:
    let b = [{ url: "http://test1." + DOMAIN_PREFIX +".mozilla.com/",
              title: "test1",
              tags:["testing", "deleteme"] },
             { url: "http://test2." + DOMAIN_PREFIX +".mozilla.com/",
               title: "test2",
               tags: ["testing", "deleteme"]}
            ];

    this.places.bookmark.create(b, callbackC);

    function callbackC(createdBookmarks) {
      self.assert(createdBookmarks.length == 2);
    }

  },

  testIncorrectTagsInput: function(self) {
    let domain_prefix = Date.now();
    let test_url = "http://biff." + domain_prefix + ".mozilla.com/";

    let bObj = { url: test_url,
                 title: "my mozilla " + domain_prefix,
                 tags: 67890
               };

    let caught = false, message;

    try {
      // no callback needed to test for an error
      this.places.bookmark.create(bObj);
    }
    catch(ex) {
      caught = true;
      message = ex.message;
    }
    self.assert(caught);
    self.assert(message == "Tags input is invalid. String or Array required");
  }
};
