function createToolbar() {
  let fakeContext = { id: Date.now() };
  return new Toolbar(fakeContext);
}

var ToolbarTests = {
  testNavigationButtonSimple: function(self) {
    // This was taken from test-status-bar-panel.js
    // It's pretty hacky...
    var browserCount = 0;
    var bw = new BrowserWatcher({
      onLoad: function(window) {
        if (!window.document.getElementById("nav-bar").hidden)
          browserCount += 1;
      }
    });
    bw.unload();

    // Only run the test if we have browser windows
    if (!browserCount)
      return;

    let testButton = {
      id: "testButton_navigation",
      label: "Testing123",
      image: "http://www.mozilla.org/favicon.ico",
      onclick: function(event) {
        console.log("Wut Wut!");
      },
      tooltip: "I AM TOOLTIP TEXT"
    };

    let toolbar = createToolbar();

    toolbar.navigation.append(testButton);

    // Look at each window to see if the button exists, manually searching for the ID
    let buttonID = ["jetpackbutton", 0, toolbar._featureContext.id, testButton.id].join("-");
    let hasButton = true;
    bw = new BrowserWatcher({
      onLoad: function(window) {
        //XXXzpao We could make this more complete by checking attributes
        if (!window.document.getElementById("PersonalToolbar").hidden)
          hasButton = hasButton && window.document.getElementById(buttonID);
      }
    });
    bw.unload();
    self.assert(hasButton, "Button has been added to each window.");

    // Cleanup so we don't leak
    toolbar.unload();
  },

  testBookmarksButtonSimple: function(self) {
    // This was taken from test-status-bar-panel.js
    // It's pretty hacky...
    var browserCount = 0;
    var bw = new BrowserWatcher({
      onLoad: function(window) {
        if (!window.document.getElementById("PersonalToolbar").hidden)
          browserCount += 1;
      }
    });
    bw.unload();

    // Only run the test if we have browser windows
    if (!browserCount)
      return;

    let testButton = {
      id: "testButton_bookmark",
      label: "Mozilla.org",
      image: "http://www.mozilla.org/favicon.ico",
      url: "http://www.mozilla.org/",
      tooltip: "Visit the website. It's awesome!"
    };

    let toolbar = createToolbar();

    toolbar.bookmarks.append(testButton);

    // Look at each window to see if the button exists, manually searching for the ID
    let buttonID = ["jetpackbutton", 1, toolbar._featureContext.id, testButton.id].join("-");
    let hasButton = true;
    bw = new BrowserWatcher({
      onLoad: function(window) {
        //XXXzpao We could make this more complete by checking attributes
        if (!window.document.getElementById("PersonalToolbar").hidden)
          hasButton = hasButton && window.document.getElementById(buttonID);
      }
    });
    bw.unload();
    self.assert(hasButton, "Button has been added to each window.");

    // Cleanup so we don't leak
    toolbar.unload();
  },

  testSameToolbar: function(self) {
    let toolbar = createToolbar();

    self.assert(toolbar.navigation === toolbar.navigation,
                "Navigation toolbars created in the same context are not the same");
    self.assert(toolbar.bookmarks === toolbar.bookmarks,
                "Bookmark toolbars created in the same context are not the same");
    toolbar.unload();
  },

  testAddButtonsWithSameIdToSameContext: function(self) {
    let testButton1 = {
      id: "testButton_sameId",
      onclick: function(event) { console.log("testButton1") }
    };
    let testButton2 = {
      id: "testButton_sameId",
      onclick: function(event) { console.log("testButton2") }
    };

    let toolbar = createToolbar();

    let gotError = false;
    toolbar.navigation.append(testButton1);
    try {
      toolbar.navigation.append(testButton2);
    } catch (e) {
      //XXXzpao We should probably do explicit error matching
      gotError = true;
    }
    self.assert(gotError,
                "We shouldn't be allowed to add buttons with the same id to " +
                "the same toolbar in the same context");

    toolbar.unload();
  },

  testAddButtonsWithSameIdToDifferentContexts: function(self) {
    let testButton1 = {
      id: "testButton_sameId",
      onclick: function(event) { console.log("testButton1") }
    };
    let testButton2 = {
      id: "testButton_sameId",
      onclick: function(event) { console.log("testButton2") }
    };

    let toolbar1 = createToolbar();
    let toolbar2 = createToolbar();

    let gotError = false;
    toolbar1.navigation.append(testButton1);
    try {
      toolbar2.navigation.append(testButton2);
    } catch (e) {
      gotError = true;
    }
    self.assert(!gotError,
                "We should be allowed to add buttons with the same id to the " +
                "same toolbar in different contexts");

    toolbar1.unload();
    toolbar2.unload();
  },

  //XXXzpao remove was implemented but I haven't had a chance to write the proper tests.
  // testRemoveNotImplemented: function(self) {
  //   let toolbar = createToolbar();
  //   let testButton = {
  //     id: "testButton_remove",
  //     onclick: function(event) { console.log("testButton") },
  //     url: "http://example.com"
  //   };

  //   let gotError = false;
  //   toolbar.navigation.append(testButton);
  //   try {
  //     toolbar.navigation.remove(testButton.id);
  //   } catch (e) {
  //     if (e.result == Components.results.NS_ERROR_NOT_IMPLEMENTED)
  //       gotError = true;
  //   }
  //   self.assert(gotError,
  //               "toolbar.navigation.remove should throw NS_ERROR_NOT_IMPLEMENTED");

  //   toolbar.unload();
  // }

};
